/***************************************************************************
                          surfxp.cpp  -  description
                             -------------------
    begin                : Die M�  5 19:59:18 CET 2002
    copyright            : (C) 2002 by Jrgen B�m
    email                : aviduratas@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it.                                                                   *
 *                                                                         *
 ***************************************************************************/

#define HAVE_CONFIG_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <QAction>
#include <QFileDialog>
#include <QStatusBar>
#include <QPrinter>
#include <QtWidgets>

#include "surfxp.h"
#include "filesave.xpm"
#include "fileopen.xpm"
#include "filenew.xpm"

SurfxpApp::SurfxpApp()
{
  setWindowTitle(tr("Surfxp " VERSION));

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initActions();
  initMenuBar();
  initToolBar();
  initStatusBar();

  initDoc();
  initView();


  //viewToolBar->setEnabled(true);
  //viewStatusBar->setEnabled(true);

	viewToolBar->setChecked(true);
	viewStatusBar->setChecked(true);

	resize(600, 600);

}

SurfxpApp::~SurfxpApp()
{
}

/** initializes all QActions of the application */
void SurfxpApp::initActions(){

  QPixmap openIcon, saveIcon, newIcon;
  newIcon = QPixmap(filenew);
  openIcon = QPixmap(fileopen);
  saveIcon = QPixmap(filesave);


  //fileNew = new QAction(tr("New File"), newIcon, tr("&New"), QAccel::stringToKey(tr("Ctrl+N")), this);
  fileNew = new QAction(newIcon, tr("New File"), this);
  fileNew->setStatusTip(tr("Creates a new document"));
  fileNew->setWhatsThis(tr("New File\n\nCreates a new document"));
  connect(fileNew, SIGNAL(triggered()), this, SLOT(slotFileNew()));

  //fileOpen = new QAction(tr("Open File"), openIcon, tr("&Open..."), 0, this);
  fileOpen = new QAction(openIcon, tr("Open File"), this);
  fileOpen->setStatusTip(tr("Opens an existing document"));
  fileOpen->setWhatsThis(tr("Open File\n\nOpens an existing document"));
  connect(fileOpen, SIGNAL(triggered()), this, SLOT(slotFileOpen()));

  //fileSave = new QAction(tr("Save File"), saveIcon, tr("&Save"), QAccel::stringToKey(tr("Ctrl+S")), this);
  fileSave = new QAction(saveIcon, tr("Save File"), this);
  fileSave->setStatusTip(tr("Saves the actual document"));
  fileSave->setWhatsThis(tr("Save File.\n\nSaves the actual document"));
  connect(fileSave, SIGNAL(triggered()), this, SLOT(slotFileSave()));

  //fileSaveAs = new QAction(tr("Save File As"), tr("Save &as..."), 0, this);
  fileSaveAs = new QAction(tr("Save File As"), this);
  fileSaveAs->setStatusTip(tr("Saves the actual document under a new filename"));
  fileSaveAs->setWhatsThis(tr("Save As\n\nSaves the actual document under a new filename"));
  connect(fileSaveAs, SIGNAL(triggered()), this, SLOT(slotFileSave()));

  //fileClose = new QAction(tr("Close File"), tr("&Close"), QAccel::stringToKey(tr("Ctrl+W")), this);
  fileClose = new QAction(tr("Close File"), this);
  fileClose->setStatusTip(tr("Closes the actual document"));
  fileClose->setWhatsThis(tr("Close File\n\nCloses the actual document"));
  connect(fileClose, SIGNAL(triggered()), this, SLOT(slotFileClose()));

  //filePrint = new QAction(tr("Print File"), tr("&Print"), QAccel::stringToKey(tr("Ctrl+P")), this);
  filePrint = new QAction(tr("Print File"), this);
  filePrint->setStatusTip(tr("Prints out the actual document"));
  filePrint->setWhatsThis(tr("Print File\n\nPrints out the actual document"));
  connect(filePrint, SIGNAL(triggered()), this, SLOT(slotFilePrint()));

  //fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit = new QAction(tr("Exit"), this);
  fileQuit->setStatusTip(tr("Quits the application"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  connect(fileQuit, SIGNAL(triggered()), this, SLOT(slotFileQuit()));

  //controlOn = new QAction(tr("Control On"), tr("Control &On"), QAccel::stringToKey(tr("Ctrl+O")), this);
  controlOn = new QAction(tr("Control On"), this);
  controlOn->setStatusTip(tr("Shows control window"));
  controlOn->setWhatsThis(tr("Control On\n\nShows control window"));
  connect(controlOn, SIGNAL(triggered()), this, SLOT(slotControlOn()));

  //controlOff = new QAction(tr("Control Off"), tr("Control O&ff"), QAccel::stringToKey(tr("Ctrl+F")), this);
  controlOff = new QAction(tr("Control Off"), this);
  controlOff->setStatusTip(tr("Hides control window"));
  controlOff->setWhatsThis(tr("Control Off\n\nHides Control Window"));
  connect(controlOff, SIGNAL(triggered()), this, SLOT(slotControlOff()));


  //viewToolBar = new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
  viewToolBar = new QAction(tr("Toolbar"), this );
  viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
  viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
	viewToolBar->setCheckable(true);
  connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));

  //viewStatusBar = new QAction(tr("Statusbar"), tr("&Statusbar"), 0, this, 0, true);
  viewStatusBar = new QAction(tr("Statusbar"), this);
  viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
  viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
	viewStatusBar->setCheckable(true);
  connect(viewStatusBar, SIGNAL(toggled(bool)), this, SLOT(slotViewStatusBar(bool)));

  //helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
  helpAboutApp = new QAction(tr("About"), this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  connect(helpAboutApp, SIGNAL(triggered()), this, SLOT(slotHelpAbout()));

}

void SurfxpApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR
  fileMenu = menuBar()->addMenu(tr("&File"));
  controlMenu = menuBar()->addMenu(tr("&Control"));
  viewMenu = menuBar()->addMenu(tr("&View"));
  menuBar()->addSeparator();
  helpMenu = menuBar()->addMenu(tr("&Help"));

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
	fileMenu->addAction(fileNew);
	fileMenu->addAction(fileOpen);
	fileMenu->addAction(fileClose);

  fileMenu->addSeparator();
	
	fileMenu->addAction(fileSave);
	fileMenu->addAction(fileSaveAs);

  fileMenu->addSeparator();
	
	fileMenu->addAction(filePrint);

  fileMenu->addSeparator();

	fileMenu->addAction(fileQuit);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu

	controlMenu->addAction(controlOn);
	controlMenu->addAction(controlOff);


  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  

	// viewMenu->setCheckable(true);

	viewMenu->addAction(viewToolBar);
	viewMenu->addAction(viewStatusBar);


  ///////////////////////////////////////////////////////////////////
  // EDIT YOUR APPLICATION SPECIFIC MENUENTRIES HERE

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu

	helpMenu->addAction(helpAboutApp);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION

}

void SurfxpApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  fileToolbar = addToolBar("file operations");
	fileToolbar->addAction(fileNew);
	fileToolbar->addAction(fileOpen);
	fileToolbar->addAction(fileSave);

  fileToolbar->addSeparator();
  // QWhatsThis::whatsThisButton(fileToolbar);

}

void SurfxpApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->showMessage(tr("Ready."), 2000);
}

void SurfxpApp::initDoc()
{
   doc=new SurfxpDoc();
}

void SurfxpApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  view=new SurfxpView(this, doc);
  setCentralWidget(view);
}

bool SurfxpApp::queryExit()
{
  int exit=QMessageBox::information(this, tr("Quit..."),
                                    tr("Do you really want to quit?"),
                                    QMessageBox::Ok, QMessageBox::Cancel);

  if (exit==1)
  {

  }
  else
  {

  };

  return (exit==1);
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void SurfxpApp::slotFileNew()
{
  statusBar()->showMessage(tr("Creating new file..."));
  doc->newDoc();
  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotFileOpen()
{
  statusBar()->showMessage(tr("Opening file..."));

  //QString fileName = QFileDialog::getOpenFileName(0,0,this);

	QString fileName = QFileDialog::getOpenFileName(this,
    tr(""), ".", tr(""));

  if (!fileName.isEmpty())
  {
    doc->load(fileName);
    setWindowTitle(fileName);
    QString message=tr("Loaded document: ")+fileName;
    statusBar()->showMessage(message, 2000);
  }
  else
  {
    statusBar()->showMessage(tr("Opening aborted"), 2000);
  }
}


void SurfxpApp::slotFileSave()
{
  statusBar()->showMessage(tr("Saving file..."));
  doc->save();
  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotFileSaveAs()
{
  statusBar()->showMessage(tr("Saving file under new filename..."));

  QString fn = QFileDialog::getSaveFileName(this,
    tr(""), ".", tr(""));


  // QString fn = QFileDialog::getSaveFileName(0, 0, this);
  if (!fn.isEmpty())
  {
    doc->saveAs(fn);
  }
  else
  {
    statusBar()->showMessage(tr("Saving aborted"), 2000);
  }

  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotFileClose()
{
  statusBar()->showMessage(tr("Closing file..."));

  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotFilePrint()
{
  statusBar()->showMessage(tr("Printing..."));
  QPrinter printer;
  //if (printer.setup(this))
  {
    QPainter painter;
    painter.begin(&printer);

    ///////////////////////////////////////////////////////////////////
    // TODO: Define printing by using the QPainter methods here

    painter.end();
  };

  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotFileQuit()
{
  statusBar()->showMessage(tr("Exiting application..."));
  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if(doc->isModified())
  {
    if(queryExit())
    {
      qApp->quit();
    }
    else
    {

    };
  }
  else
  {
    qApp->quit();
  };

  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotControlOn()
{
  statusBar()->showMessage(tr("Showing control window..."));

  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotControlOff()
{
  statusBar()->showMessage(tr("Hiding control window..."));

  statusBar()->showMessage(tr("Ready."));
}



void SurfxpApp::slotViewToolBar(bool toggle)
{
  statusBar()->showMessage(tr("Toggle toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off

  if (toggle== false)
  {
    fileToolbar->hide();
  }
  else
  {
    fileToolbar->show();
  };

  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotViewStatusBar(bool toggle)
{
  statusBar()->showMessage(tr("Toggle statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off

  if (toggle == false)
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  statusBar()->showMessage(tr("Ready."));
}

void SurfxpApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About..."),
                      tr("Surfxp\nVersion " VERSION "\n(c) 2002 by Jürgen Böhm") );
}

