/***************************************************************************
                          surfxpview.cpp  -  description
                             -------------------
    begin                : Die M�  5 19:59:18 CET 2002
    copyright            : (C) 2002 by Jrgen B�m
    email                : aviduratas@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it.                                                                   *
 *                                                                         *
 ***************************************************************************/


#include "surfxpview.h"

const GLfloat Pi = 3.14159265;


SurfxpView::SurfxpView(QWidget *parent, SurfxpDoc *doca) : QOpenGLWidget(parent), doc(doca)
{
  /** connect doc with the view*/
  connect(doc, SIGNAL(documentChanged()), this, SLOT(slotDocumentChanged()));
}

SurfxpView::~SurfxpView()
{
}

void SurfxpView::slotDocumentChanged()
{
  //TODO update the view

	repaint ();

}

void SurfxpView::initializeGL () {

//glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );

glEnable ( GL_DEPTH_TEST );

glEnable ( GL_LIGHTING );



glClearColor ( 1.0, 1.0, 1.0, 1.0 );

glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

glMatrixMode ( GL_MODELVIEW );

glLoadIdentity ();

glGetFloatv ( GL_MODELVIEW_MATRIX, rotmatrix );

}


void SurfxpView::paintGL () {


glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

if (doc->grid.active ()) {


	glMatrixMode ( GL_MODELVIEW );
			
	if ( oldx != -1 ) {
	
		glLoadIdentity ();
			
		glRotatef ( angle, vturn.x, vturn.y, vturn.z );
		glMultMatrixf ( rotmatrix );
		
		glGetFloatv ( GL_MODELVIEW_MATRIX, rotmatrix );
		
		glLoadIdentity ();
		
		glTranslatef ( 0, 0, -15 );
		
		glMultMatrixf ( rotmatrix );
			
	};		
	
	GLfloat ambientfront[] = { 0.2, 0.0, 0.0, 1.0 };
	GLfloat diffusefront[] = { 0.8, 0.0, 0.0, 1.0 };
	
	GLfloat ambientback[] = { 0.0, 0.0, 0.2, 1.0 };
	GLfloat diffuseback[] = { 0.0, 0.0, 0.8, 1.0 };
	
	GLfloat specular [] = { 1.0, 1.0, 1.0, 1.0 };
	
	
	glMaterialfv ( GL_FRONT, GL_AMBIENT, ambientfront );
	glMaterialfv ( GL_FRONT, GL_DIFFUSE, diffusefront );
	glMaterialfv ( GL_FRONT, GL_SPECULAR, specular );
	
	
	glMaterialfv ( GL_BACK, GL_AMBIENT, ambientback );
	glMaterialfv ( GL_BACK, GL_DIFFUSE, diffuseback );
	glMaterialfv ( GL_BACK, GL_SPECULAR, specular );
	
	
	
	glMaterialf ( GL_FRONT_AND_BACK, GL_SHININESS, 100.0 );
	
	glLightModeli ( GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE );
	
	GLfloat global_ambient[] = { 0.9, 0.9, 0.9, 1.0 };
	
	GLfloat light0pos[] = { 0.0, 0.0, 10.0, 1.0 };
	
	
	glEnable ( GL_LIGHT0 );
	
	
	glPushMatrix ();
	glLoadIdentity ();
	
	glLightfv ( GL_LIGHT0, GL_POSITION, light0pos );
	
	
	glPopMatrix ();
	
	glLightfv ( GL_LIGHT0, GL_AMBIENT, global_ambient );
	glLightfv ( GL_LIGHT0, GL_DIFFUSE, global_ambient );
	glLightfv ( GL_LIGHT0, GL_SPECULAR, global_ambient );	
	
	
	glShadeModel ( GL_SMOOTH );	
	
	glBegin ( GL_TRIANGLES );
		
	for (int i = 0; i < doc->grid.rows () - 1; i++ ) {	
		
		Point P0 = doc->grid ( i, 0 );
		
		Point N0 = doc->gridnormal ( i, 0 );
		
		Point P1 = doc->grid ( i+1, 0);
		
		Point N1 = doc->gridnormal ( i+1, 0 );
	
		for ( int j = 0; j < doc->grid.cols () - 1; j++ ) {
				
			Point P2 = doc->grid ( i, j+1 );
			
			Point N2 = doc->gridnormal ( i, j+1 );

			glNormal3f ( N0.x, N0.y, N0.z );
			glVertex3f ( P0.x, P0.y, P0.z );
			glNormal3f ( N1.x, N1.y, N1.z );
			glVertex3f ( P1.x, P1.y, P1.z );
			glNormal3f ( N2.x, N2.y, N2.z );
			glVertex3f ( P2.x, P2.y, P2.z );
		
			Point P3 = doc->grid ( i+1, j+1 );
			
			Point N3 = doc->gridnormal ( i+1, j+1 );
		
			glNormal3f ( N1.x, N1.y, N1.z );
			glVertex3f ( P1.x, P1.y, P1.z );
			glNormal3f ( N3.x, N3.y, N3.z );
			glVertex3f ( P3.x, P3.y, P3.z );
			glNormal3f ( N2.x, N2.y, N2.z );			
			glVertex3f ( P2.x, P2.y, P2.z );
					
			P0 = P2;
			P1 = P3;
			
			N0 = N2;
			N1 = N3;			
		};
	};
		
	glEnd ();
	
/*	The following code is for showing the normal-field of the surface

	glBegin ( GL_LINES );
	
	for ( int j = 0; j < doc->grid.cols (); j++ ) {
	
	for ( int i = 0; i < doc->grid.rows (); i++ ) {
	
		Point P0 = doc->grid (i,j);
		
		Point N0 = doc->gridnormal ( i, j );
		
		Point P1;
		
		vecplus ( P0, N0, P1 );
		
		glVertex3f ( P0.x, P0.y, P0.z );
		
		glVertex3f ( P1.x, P1.y, P1.z );
		
	
	}; };
	
	
	glEnd ();
*/	
	
	glFlush ();
	
};
	
}


void SurfxpView::resizeGL ( int w, int h ) {

wd = w;
ht = h;

glViewport ( 0, 0, w, h );



glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );


glMatrixMode ( GL_PROJECTION );

glLoadIdentity ();

GLfloat xs, ys, zmin, zmax;

xs = 0.036 / 2;

ys = ((float) (h)) / w * xs;

glFrustum ( -xs, xs, -ys, ys, 0.05, 100.0 );


glMatrixMode ( GL_MODELVIEW );


repaint ();


}

#define min(x,y) ((x < y) ? x : y)

void trackballvec ( int x, int y, int w, int h, Point & p ) {

GLfloat xx, yy, d;

xx = x - w / 2.0;
yy = h / 2.0 - y;

d = sqrt ( xx*xx + yy*yy );

GLfloat r = min ( w / 2.0, h / 2.0 );

if ( d > r ) {

	xx = xx * r/d;
	yy = yy * r/d;
	
};


GLfloat zz2 = ((r*r) - (xx*xx + yy*yy) );

if (zz2 < 0)
	zz2 = 0;


GLfloat zz = sqrt ( zz2 );

p.x = xx/r;
p.y = yy/r;
p.z = zz/r;

}


void SurfxpView::mouseMoveEvent ( QMouseEvent * e ) {

int newx = e->x();
int newy = e->y();

Point vnew, p0;

trackballvec ( newx, newy, wd, ht, vnew );

cross ( p0, vold, vnew, vturn );


angle = asin ( sqrt ( dotprod ( vturn, vturn ) ) );

angle = 180.0 * angle / Pi;


vold = vnew;

oldx = newx;
oldy = newy;

repaint ();


}


		
void SurfxpView::mousePressEvent ( QMouseEvent * e ) {

oldx = e->x();
oldy = e->y();

trackballvec ( oldx, oldy, wd, ht, vold );

}


		
void SurfxpView::mouseReleaseEvent ( QMouseEvent * e ) {

oldx = -1;
oldy = -1;

}


















